from abc import ABCMeta, abstractmethod
from collections.abc import Iterable

__all__ = [
    "BaseConfig",
]


class BaseConfig(metaclass=ABCMeta):
    """Base Config"""

    def __init__(
        self,
        config_types,
        name="base-config",
    ):
        self.config_types = config_types
        if not isinstance(config_types, Iterable):
            if not isinstance(config_types, (list, tuple)):
                raise AssertionError(
                    f"Expected a valid config tpye of tuple or list, but got {type(config_types)}"
                )

        if "base" in name:
            raise AssertionError(
                f"Expected a valid name, but received base class name: {name}"
            )
        self.name = name

        self._parse_types()

    def _parse_types(self):
        """config types builder"""
        for __o in self.config_types:
            key = __o.key
            setattr(self, key, __o)

    def get_type(self, key):
        """getter method for snagging typed values"""
        return getattr(self, key)

    def set_type(self, __o, value):
        """setter to change a mutable attribute"""
        if __o.is_mutable:
            setattr(self, __o.key, value)
        else:
            print(
                f"Config got a request to change a immutable type {type.key} with {value}. It is not allowed"
            )

    @abstractmethod
    def __str__(self):
        """to string method"""
        pass



from walle.core import UserNameType

import pytest


@pytest.mark.parametrize(
    "test_input,expected",
    [
        (
            {
                "key": "username",
                "value": "rube",
                "name": "dummy-type",
            },
            {
                "key": "username",
                "value": "rube",
                "name": "dummy-type",
            },
        ),
    ],
)
def test_eval(test_input, expected):
    got = UserNameType(**test_input)
    assert got.name == expected["name"]
    assert got.key == expected["key"]
    assert got.value == expected["value"]
    expected = UserNameType(**expected)
    assert got == expected
    assert str(got) == expected.key



from behave import given
from behave import when
from behave import then

import importlib


@given("we have a {type_name} with {key} and {value}")
def build_type(context, type_name, key, value):
    pkg = importlib.import_module("walle")
    type_o = getattr(pkg, type_name)
    __o = type_o(key=key, value=value)
    context.__o = __o


@when("we append {string}")
def when_types_action(context, string):
    context.to_string += " " + string.strip()


@then("we can extract a to_string")
def extract_string(context):
    to_string = str(context.__o)
    context.to_string = to_string.strip()


@then("we expect a word count of {count}")
def expect_type_return(context, count):
    assert len(context.to_string.split(" ")) == int(count)



# -*- coding: utf-8 -*-

# =================================================================================
# Authors:
#       Jon Perry
#       Joshua Oliveira
# Email:
#       jon@jonperry.dev
#       Joshua.oliveiramartin@gmail.com
# =================================================================================

from pathlib import Path
from setuptools import setup, find_packages

try:
    version_path = Path(__file__).parent / "walle" / "VERSION"
    version = version_path.read_text().strip()
except FileNotFoundError:
    version = "0.0.0"

required_files = "requirements/REQUIREMENTS"
REQUIREMENTS = [req.strip() for req in open(required_files, "r").readlines()]


def setup_package():
    setup(
        name="discord-walle",
        version=version,
        description="General Discord Utility Bot",
        author=[
            "Jon Perry",
            "Josh Oliveira",
        ],
        license="Unlicense",
        packages=find_packages(),
        include_package_data=True,
        install_requires=REQUIREMENTS,
        python_requires="~=3.8",
    )


if __name__ == "__main__":
    setup_package()



from .base import BaseConfig
from walle.core import constants

__all__ = [
    "WelcomeConfig",
]


class WelcomeConfig(BaseConfig):
    """"""

    def __init__(
        self,
        config_types,
        name="welcome-config",
    ):
        super().__init__(
            config_types=config_types,
            name=name,
        )

    def _welcome_message(self):
        __o = self.get_type(constants.USERNAME)
        return f"Welcome to the server, {__o.value}!"

    def __str__(self):
        return self._welcome_message()
